import os as _os


def _set_credentials():
    try:
        home = _os.path.expanduser("~")
        with open(home + '/.pypi_umihico_env', 'r') as f:
            for line in f.readlines():
                key, password = line.split(',')
                _os.environ[key] = password.replace('\n', '')
    except Exception as e:
        pass


_set_credentials()


def _set_env_value(getenv_key, optional_value=None):
    """
    example
    def send_line(message, receiver_api_key=None):
        receiver_api_key = _set_env_value('line_api_key', receiver_api_key)
    """
    value = _os.getenv(getenv_key) or optional_value
    if not value:
        raise Exception(f"{getenv_key} is not defined")
    return value


from . import api
from . import aws
from . import hash_
from . import io_
from . import listing
from . import scraping
from . import string
from . import zip
from . import xlsx



from lxml.etree import ElementBase as _ElementBase
from lxml.html import fromstring as _fromstring


def elements_to_strings_list(lxml_elements):
    lxml_elements = lxml_elements if isinstance(
        lxml_elements, list) else [lxml_elements, ]
    return [str(element.text_content() if isinstance(
        element, _ElementBase) else element) for element in lxml_elements]


def vaild_xpath(xpath):
    emptylxml = _fromstring(
        '<html><head><title></title></head><body></body></html>')
    try:
        emptylxml.xpath(xpath)
        return True
    except Exception as e:
        return False



import xlsxwriter as _xlsxwriter
import xlrd as _xlrd


def load_xlsx(filename):
    xl_workbook = _xlrd.open_workbook(filename)
    xl_sheet = xl_workbook.sheet_by_index(0)
    nrows = xl_sheet.nrows
    ncols = xl_sheet.ncols
    rows = [[str(xl_sheet.cell(row_index, col_index).value)
             for col_index in range(0, ncols)] for row_index in range(0, nrows)]
    return rows


def to_xlsx(filename, list_of_list):
    workbook = _xlsxwriter.Workbook(filename)
    worksheet = workbook.add_worksheet()
    for row_index, row in enumerate(list_of_list):
        for col_index, cell in enumerate(row):
            worksheet.write(row_index, col_index, cell)
    workbook.close()





import requests as _requests
import json as _json
import functools as _functools
import pprint as _pprint
import traceback as _traceback
from ..zip import compress_text as _compress_text
from ..zip import decompress_text as _decompress_text
import ast as _ast
import os as _os


def _get_content_type():
    return _os.getenv('apigateway_decorator_Content_Type', "text/plain")


def set_content_type(content_type="text/html"):
    _os.environ['apigateway_decorator_Content_Type'] = content_type


def event2args(event):
    compressed_text = event['body']
    if compressed_text:
        text = _decompress_text(compressed_text)
        return _ast.literal_eval(text)
    else:
        return []


def args2payload(*args):
    compressed_text = _compress_text(str(args))
    payload = compressed_text
    return payload


def trigger_via_apigateway(url, api_key=None, payload=None):
    method = "POST" if payload else "GET"
    kw = {}
    if api_key:
        kw['headers'] = {'x-api-key': api_key, }
    if payload:
        kw['data'] = _json.dumps(payload)
    return _requests.request(method, url, **kw)


def apigateway_decorator(lambda_handler):
    @_functools.wraps(lambda_handler)
    def lambda_handler_wrapper(event, context):
        try:
            body = str(lambda_handler(event, context))
            statusCode = 200
        except Exception as e:
            statusCode = 500
            body = "\n_____error_log_____\n" + \
                _traceback.format_exc() + "\n_____error_log_____"
            print(body)
        return {"statusCode": statusCode, "headers": {"Content-Type": _get_content_type()}, "body": str(body)}
    return lambda_handler_wrapper



import codecs as _codecs
import ast as _ast
from pprint import pformat as _pformat


def save_as_txt(filename, data, mode='w'):
    with _codecs.open(filename, mode, 'utf-8') as f:
        f.write(_pformat(data))


def load_from_txt(filename):
    with _codecs.open(filename, 'r', 'utf-8') as f:
        return _ast.literal_eval(f.read())
